/*
 * @plotXYHistogramFrame.java Version 1.1 05/25/2011
 *
 * Copyright (c) 2011 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package plot;

import javax.swing.*;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Observable;

import plot.plotXYHistogram;
import plot.plotXYStruct;
import plot.plotSymbolStruct;

/** Class plotXYHistogramFrame
 *  <p> This class will display the Cross Plot Frame
 *
 *  @version 1.1 05/25/2011
 *  @author  John R. Victorine
 */

public class plotXYHistogramFrame extends JFrame //implements ActionListener
{
  private Observable       notifier = null; // Observalbles Object
  private int              iPlot    = -1;
  private plotXYStruct     stPlot   = null;
  private plotXYHistogram  panel    = null;
  private String           sPLOT    = "";

  /** Constructor plotXYHistogramFrame()
   *  <p> This is the constructor for this class
   * @param notifier = Oberservable
   * @param iPlot    = Plot Type
   * @param stPlot   = XY Plot Data Structure\
   * @param sPLOT    = Type of Plot, "2D" or "3D"
   */

  public plotXYHistogramFrame( Observable notifier,
                               int iPlot, plotXYStruct stPlot,
                               String sPLOT)
  {
    try
    {
	  this.notifier = notifier;
	  this.iPlot    = iPlot;
      this.stPlot   = stPlot;
      this.sPLOT    = new String( sPLOT );

      jbInit();

      addWindowListener( new plotXYHistogramFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    JScrollPane jScrollPane = new JScrollPane();

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle("KS Earthquake Histogram Plot");

    panel = new plotXYHistogram(iPlot, stPlot, sPLOT);

    this.getContentPane().add(jScrollPane, BorderLayout.CENTER);
    jScrollPane.getViewport().add(panel, null);

    // Add a Window Listener to close dialog

    this.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e) { close(); }
    });

    // Set Dimension of Frame and Display it.

    if (sPLOT.equals("2D"))
    {
      this.setSize(new Dimension(900, 700));
    }
    else
    {
      this.setSize(new Dimension(900, 900));
    }

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
//    this.setLocation(3*(d.width - this.getSize().width) / 4,
//                     (d.height - this.getSize().height) / 2);
//    this.setLocation( 3*(d.width - this.getSize().width) / 4,
//                      3*(d.height - this.getSize().height)/4);

    this.setLocation( (d.width - this.getSize().width) -10, 10);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** Method refresh()
   * <p> This method will refresh the plot if it exists
   */

  public void refresh() { panel.repaint(); }

  /** METHOD plotXYHistogramFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class plotXYHistogramFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  if (notifier != null)
        notifier.notifyObservers(new String("Close Plot Control"));
    }
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (panel != null) image = panel.getImage();

    return (image);
  }

  /* ================================================================== *
   * -------------------------- SET METHODS --------------------------- *
   * ================================================================== */

  /** Method setLegend()
   *  <p> The Legend for the XY Plot.
   *  @param iLegend  - Number of Legend Rows.
   *  @param stLegend - Array of Legends.
   */

  public void setLegend(int iLegend, plotSymbolStruct stLegend[])
  {
    panel.setLegend(iLegend, stLegend);
  }

  /** Method setData()
   *  <p> This is the Constructor for this class.
   *  @param stPlot - Plot Data Structure.
   */

  public void setData(plotXYStruct stPlot)
  {
    if (panel != null)  { panel.setData(stPlot); }
  }

  /** Method setTriangle()
   * <p> This method will set the triangle data
   * @param iData      = The total number points
   * @param sLithology = The lithology values
   * @param dMat       = Rhomaa-Umaa values
   */

  public void setTriangle(int iData, String sLithology[], double dMat[][])
  {
    if (panel != null) { panel.setTriangle(iData, sLithology, dMat); }
  }

  /** Method setFlag()
   *  <p> Set a Flag to filter extra data or not.
   *  @param bFlag = true to filter data or false not to filter extra data.
   */

  public void setFlag(boolean bFlag) { panel.setFlag(bFlag); }

  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText) { panel.setTitle1(sText); }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText) { panel.setTitle2(sText); }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText) { panel.setTitle3(sText); }

  /** Method setXAxis()
   * <p> This method will reset the X-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setXAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setXAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setYAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setZAxis()
   * <p> This method will reset the Z-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setZAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setZAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  public void close()
  {
    if (stPlot != null)
      stPlot.delete();
    stPlot = null;

    if (panel != null)
      panel.close();
    panel     = null;

    sPLOT = null;

    dispose();
  }
}

/*
 *  @version 1.1 05/25/2011
 *  @author  John Victorine
 */
