/*
 * @quakePlotTimeMagDataFrame.java Version 1.1 08/06/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import gui.guiSearchDirectoryFrame;    // File Search Dialog

import quake.plot.quakePlotTimeMagData;
import quake.quakeSeisStruct;

import plot.plotSymbolStruct;

/** Class quakePlotTimeMagDataFrame
 *  <p> This class will display the 2D Plot Frame
 *
 *  @version 1.1 08/06/2015
 *  @author  John R. Victorine
 */

public class quakePlotTimeMagDataFrame extends JFrame implements ActionListener, Observer
{
  private static final int _START = 0;
  private static final int _END   = 1;

//  private static final int START   = 0;
//  private static final int END     = 1;
//  private static final int MINIMUM = 2;
//  private static final int MAXIMUM = 3;

  private static final int YEAR    = 0;
  private static final int MONTH   = 1;
  private static final int DAY     = 2;
  private static final int HOUR    = 3;
  private static final int MIN     = 4;
  private static final int SEC     = 5;

  // Input Variables

  private quakeSeisStruct         stData   = null;

  // Global Variables

  private Observable              notifier = null; // Observalbles Object
//  private int                     iPlot    = -1;
  private quakePlotTimeMagData    panel    = null; // Time Magnitude 2D Plot

  // Save Path & Filename

  private String                  sPath   = new String("");
  private String                  sName   = new String("");
  private guiSearchDirectoryFrame pSearch = null; // Search for Directory to Save PNG

  private Timestamp               tsStart = null; // Starting Timestamp
  private Timestamp               tsEnd   = null; // Ending Timestamp

  private int                     iYear   = 0;
  private int                     iMonth  = 0;

  private int                     iYStart = 0;
  private int                     iYEnd   = 0;
  private int                     iMStart = 0;
  private int                     iMEnd   = 0;

  private String                  sTitle  = "";

  private String                  sTitle1 = "Kansas Geological Survey";
  private String                  sTitle2 = "Kansas Earthquakes";
  private String                  sTitle3 = "";

  // X-Axis Values.

  public int          iXLog       = 0;   // X-Axis Semilog Indicator
                                         //   1-Semilog
                                         //   0-Linear
  public int          iXCycles    = 1;   // Number of Cycles
  public double       dXMaximum   = 0.0; // Maximum X-Axis
  public double       dXMinimum   = 0.0; // Minimum X-Axis
  public double       dXIncrement = 0.0; // Increment along the X-Axis
  public String       sXAxis      = "";  // X-Axis Label.
  private String      sXIncr      = "Increment:";


  // Y-Axis Values.

  public int          iYLog       = 0;   // Y-Axis Semilog Indicator
                                         //   1-Semilog
                                         //   0-Linear
  public int          iYCycles    = 1;   // Number of Cycles
  public double       dYMaximum   = 6.0; // Maximum Y-Axis
  public double       dYMinimum   = 0.0; // Minimum Y-Axis
  public double       dYIncrement = 1.0; // Increment along the Y-Axis
  public String       sYAxis      = "Earthquake Magnitude";  // Y-Axis Label.
  private String      sYIncr      = "Increment:";

  // Global Frame Widgets

  // .. Plot Titles

  private JTextField txtTitle1 = new JTextField();
  private JTextField txtTitle2 = new JTextField();
  private JTextField txtTitle3 = new JTextField();

  // .. X-Axis Widgets

  private JTextField txtXAxis  = new JTextField();

  private JTextField txtYYYYs  = new JTextField(); // Year Field
  private JTextField txtMMs    = new JTextField(); // Month Field
  private JTextField txtDDs    = new JTextField(); // Day Field

  private JTextField txtHHs    = new JTextField(); // 24 Hour Field
  private JTextField txtmms    = new JTextField(); // minute Field
  private JTextField txtssSs   = new JTextField(); // second Field

  private JTextField txtYYYYe  = new JTextField(); // Year Field
  private JTextField txtMMe    = new JTextField(); // Month Field
  private JTextField txtDDe    = new JTextField(); // Day Field

  private JTextField txtHHe    = new JTextField(); // 24 Hour Field
  private JTextField txtmme    = new JTextField(); // minute Field
  private JTextField txtssSe   = new JTextField(); // second Field

//  private JTextField   txtXMin    = new JTextField();
//  private JTextField   txtXMax    = new JTextField();
//  private JTextField   txtXIncr   = new JTextField();

  // .. Y-Axis Widgets

  private JTextField txtYAxis  = new JTextField();
  private JTextField txtYMin   = new JTextField();
  private JTextField txtYMax   = new JTextField();
  private JTextField txtYIncr  = new JTextField();

  // -- Menus
  // .. File Button Menu

  private JMenuItem mPDF  = null; // Create a PDF of Plot
  private JMenuItem mExit = null; // Exit Plot

  /** Constructor quakePlotTimeMagDataFrame()
   *  <p> This is the constructor for this class
   * @param stData   = Earthquake Data Structure
   */

  public quakePlotTimeMagDataFrame( quakeSeisStruct stData )
  {
	String sTitle = "";

    try
    {
	  this.stData   = stData;

	  if (stData != null)
	  {
		if ((stData.tsStart != null) && (stData.tsEnd != null))
		{
		  sTitle3 = new String(stData.tsStart.toString() + " to " + stData.tsEnd.toString());
		}
	  }

/*
      if (iPlot > 0)
      {
		if (stPlot[0] != null)
		{
		  sTitle = new String(stPlot[0].sTitle1);
		  sXAxis = new String(stPlot[0].sXAxis);
		  if (stPlot[0].iXLog == 1) { sXIncr = new String("Cycles:"); }
		  sYAxis = new String(stPlot[0].sYAxis);
		  if (stPlot[0].iYLog == 1) { sYIncr = new String("Cycles:"); }

          // Y-Axis Values.

		  iXLog       = stPlot[0].iXLog;       // X-Axis Semilog Indicator
                                               //   1-Semilog
                                               //   0-Linear
		  iXCycles    = stPlot[0].iXCycles;    // Number of Cycles
		  dXMaximum   = stPlot[0].dXMaximum;   // Maximum X-Axis
		  dXMinimum   = stPlot[0].dXMinimum;   // Minimum X-Axis
		  dXIncrement = stPlot[0].dXIncrement; // Increment along the X-Axis
		  sXAxis      = new String(stPlot[0].sXAxis); // X-Axis Label.

          // Y-Axis Values.

		  iYLog       = stPlot[0].iYLog;       // Y-Axis Semilog Indicator
                                               //   1-Semilog
                                               //   0-Linear
		  iYCycles    = stPlot[0].iYCycles;    // Number of Cycles
		  dYMaximum   = stPlot[0].dYMaximum;   // Maximum Y-Axis
		  dYMinimum   = stPlot[0].dYMinimum;   // Minimum Y-Axis
		  dYIncrement = stPlot[0].dYIncrement; // Increment along the Y-Axis
		  sYAxis      = new String(stPlot[0].sYAxis); // Y-Axis Label.
		}
	  }
*/
      jbInit();

      addWindowListener( new quakePlotTimeMagDataFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel      pnlTop      = new JPanel();
	JPanel      pnlTitles   = new JPanel();
	JPanel      pnlAxes     = new JPanel();
	JPanel      pnlXAxis    = new JPanel();
	JPanel      pnlYAxis    = new JPanel();
    JScrollPane jScrollPane = new JScrollPane();

    JMenuBar    menuBar      = new JMenuBar();
    JMenu       menuFile     = new JMenu("File");

    // Build Menus

    // -- Menu Bar

    mPDF = new JMenuItem("Create PDF Document Plot");
    mPDF.addActionListener(this);

    mExit = new JMenuItem("Exit");
    mExit.addActionListener(this);

    menuFile.add( mPDF );
    menuFile.addSeparator();
    menuFile.add( mExit );

    menuBar.add(menuFile);
    this.setJMenuBar(menuBar);

    // Create an Observable

    notifier = new quakePlotTimeMagDataFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Panel

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle( sTitle );

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    pnlTitles = plotTitlesPanel();

//    pnlAxes   = plotAxesPanel();
//    pnlAxes.setLayout(new GridLayout());
    pnlAxes.setLayout(new BorderLayout());

    pnlXAxis = plotXAxisPanel();
    pnlYAxis = plotYAxisPanel();

    // Center Panel

//    panel = new quakePlotTimeMagData(stData, );
    panel = new quakePlotTimeMagData( stData, "", "Earthquake Magnitude",
                                      tsStart, tsEnd, 0.0, 6.0, 1.0 );

    setTitle1(txtTitle1.getText());
    setTitle2(txtTitle2.getText());
    setTitle3(txtTitle3.getText());

    // Attach Widgets

    this.getContentPane().add(pnlTop,      BorderLayout.NORTH);
    pnlTop.add(pnlTitles,                  BorderLayout.NORTH);
    pnlTop.add(pnlAxes,                    BorderLayout.SOUTH);
    pnlAxes.add(pnlXAxis,                  BorderLayout.CENTER);
    pnlAxes.add(pnlYAxis,                  BorderLayout.EAST);

    this.getContentPane().add(jScrollPane, BorderLayout.CENTER);
    jScrollPane.getViewport().add(panel,   null);

    // Set Dimension of Frame and Display it.

    this.setSize(new Dimension(900, 800));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
//    this.setLocation( (d.width - this.getSize().width)/2,
    this.setLocation( (d.width - this.getSize().width) - 10,
                      (d.height - this.getSize().height)/2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** Method plotTitlesPanel()
   *  <p> This method will build the Plot Title Panels
   */

  private JPanel plotTitlesPanel()
  {
	JPanel panel           = new JPanel();
    JPanel pnlTitles       = new JPanel();
    JPanel pnlTitle1       = new JPanel();
    JPanel pnlTitle2       = new JPanel();
    JPanel pnlTitle3       = new JPanel();

    TitledBorder titledBorderTitle1 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "1st Title:");
    titledBorderTitle1.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTitle2 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "2nd Title:");
    titledBorderTitle2.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTitle3 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "3rd Title:");
    titledBorderTitle3.setTitleFont(new java.awt.Font("Dialog", 1, 11));


//    if (iPlot > 0)
//    {
//	  if (stPlot[0] != null)
//	  {
//		sTitle1 = new String("Kansas Geological Survey");
//		sTitle2 = new String("");
//		sTitle3 = new String("");
//	  }
//	}

    panel.setLayout(new BorderLayout());

    pnlTitles.setLayout(new GridLayout());

    // .. First Title Panel

    pnlTitle1.setLayout(new BorderLayout());
    pnlTitle1.setBorder(titledBorderTitle1);

    txtTitle1.setText(sTitle1);
    txtTitle1.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle1.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Second Title Panel

    pnlTitle2.setLayout(new BorderLayout());
    pnlTitle2.setBorder(titledBorderTitle2);

    txtTitle2.setText(sTitle2);
    txtTitle2.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle2.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Third Title Panel

    pnlTitle3.setLayout(new BorderLayout());
    pnlTitle3.setBorder(titledBorderTitle3);

    txtTitle3.setText(sTitle3);
    txtTitle3.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle3.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // Attach Widgets

    panel.add(pnlTitles,                  BorderLayout.NORTH);
    pnlTitles.add(pnlTitle1,              null);
    pnlTitle1.add(txtTitle1,              BorderLayout.CENTER);

    pnlTitles.add(pnlTitle2,              null);
    pnlTitle2.add(txtTitle2,              BorderLayout.CENTER);

    pnlTitles.add(pnlTitle3,              null);
    pnlTitle3.add(txtTitle3,              BorderLayout.CENTER);

    return (panel);
  }

  /** Method plotXAxisPanel()
   *  <p> This method will build the X-Axis Panel
   */

  private JPanel plotXAxisPanel()
  {
	JPanel panel             = new JPanel();
    JPanel pnlXAxis          = new JPanel();

    JPanel pnlDateTimeCenter = new JPanel();

    JPanel pnlDateTimeStart  = new JPanel();
    JPanel pnlDateTimeEnd    = new JPanel();

    JPanel pnlDateStart      = new JPanel();
    JPanel pnlYYYYs          = new JPanel();
    JPanel pnlMMs            = new JPanel();
    JPanel pnlDDs            = new JPanel();

    JPanel pnlTimeStart      = new JPanel();
    JPanel pnlHHs            = new JPanel();
    JPanel pnlmms            = new JPanel();
    JPanel pnlsss            = new JPanel();

    JPanel pnlDateEnd        = new JPanel();
    JPanel pnlYYYYe          = new JPanel();
    JPanel pnlMMe            = new JPanel();
    JPanel pnlDDe            = new JPanel();

    JPanel pnlTimeEnd        = new JPanel();
    JPanel pnlHHe            = new JPanel();
    JPanel pnlmme            = new JPanel();
    JPanel pnlsse            = new JPanel();

	String sText             = "";

    TitledBorder titledBorderXAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "X-Axis:");
    titledBorderXAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderStartDate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Start Date");
    titledBorderStartDate.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderEndDate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "End Date");
    titledBorderEndDate.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderYYYY = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "YYYY");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMM = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "MM");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderDD = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "DD");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderHH = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "HH");
    titledBorderHH.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBordermm = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "mm");
    titledBordermm.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderssS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "ss.S");
    titledBorderssS.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    // .. X-Axis Panel

    pnlXAxis.setLayout(new GridLayout());
    pnlXAxis.setBorder(titledBorderXAxis);

//    lblXAxis.setText( "X-Axis" );

//    txtXAxis.setText( sXAxis );
//    txtXAxis.setHorizontalAlignment(SwingConstants.LEFT);
//    txtXAxis.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. X-Axis Range Panel

//    pnlXAxisRange.setLayout(new GridLayout());

    // .......................................................
    // ..................... Date Start ......................
    // .......................................................

    iYear  = 0;
    iMonth = 0;

    if (stData != null)
    {
	  setDate(stData.tsStart);
	}

    pnlDateTimeStart.setLayout(new GridLayout());
    pnlDateTimeStart.setBorder(titledBorderStartDate);

    // -- Date Start Panel

    pnlDateStart.setLayout(new BorderLayout());

    // .. Year Textfield Panel

    pnlYYYYs.setLayout(new BorderLayout());
    pnlYYYYs.setBorder(titledBorderYYYY);
    pnlYYYYs.add(txtYYYYs,  BorderLayout.CENTER);

    sText = new String("2014");
    if (iYear > 0) { sText = new String(""+iYear); }

    txtYYYYs.setText( sText );
    txtYYYYs.setColumns(4);
	txtYYYYs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtYYYYs.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Month Textfield Panel

    pnlMMs.setLayout(new BorderLayout());
    pnlMMs.setBorder(titledBorderMM);
    pnlMMs.add(txtMMs,  BorderLayout.CENTER);

    sText = new String("01");
    if (iMonth > 0)
    {
	  if (iMonth > 9)
	    sText = new String("" + iMonth);
	  else
	    sText = new String("0" + iMonth);
    }

    txtMMs.setText( sText );
    txtMMs.setColumns(2);
	txtMMs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMMs.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Day Textfield Panel

    pnlDDs.setLayout(new BorderLayout());
    pnlDDs.setBorder(titledBorderDD);
    pnlDDs.add(txtDDs,  BorderLayout.CENTER);

    txtDDs.setText("01");
    txtDDs.setEditable(false);
    txtDDs.setColumns(2);
	txtDDs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtDDs.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // -- Attach YYYY,MM,DD Panels to Date Start Panel

    pnlDateStart.add(pnlYYYYs, BorderLayout.WEST);
    pnlDateStart.add(pnlMMs,   BorderLayout.CENTER);
    pnlDateStart.add(pnlDDs,   BorderLayout.EAST);

    // -- Time Start Panel

    pnlTimeStart.setLayout(new BorderLayout());

    // .. Hour Textfield Panel

    pnlHHs.setLayout(new BorderLayout());
    pnlHHs.setBorder(titledBorderHH);
    pnlHHs.add(txtHHs, BorderLayout.CENTER);

    txtHHs.setText("00");
    txtHHs.setEditable(false);
    txtHHs.setColumns(2);
	txtHHs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtHHs.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Minutes Textfield Panel

    pnlmms.setLayout(new BorderLayout());
    pnlmms.setBorder(titledBordermm);
    pnlmms.add(txtmms, BorderLayout.CENTER);

    txtmms.setText("00");
    txtmms.setEditable(false);
    txtmms.setColumns(2);
	txtmms.setHorizontalAlignment(SwingConstants.TRAILING);
	txtmms.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Seconds Textfield Panel

    pnlsss.setLayout(new BorderLayout());
    pnlsss.setBorder(titledBorderssS);
    pnlsss.add(txtssSs, BorderLayout.CENTER);

    txtssSs.setText("00.0");
    txtssSs.setEditable(false);
    txtssSs.setColumns(4);
	txtssSs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtssSs.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // -- Attach HH,mm,ss.SS Panels to Date Start Panel

    pnlTimeStart.add(pnlHHs, BorderLayout.WEST);
    pnlTimeStart.add(pnlmms, BorderLayout.CENTER);
    pnlTimeStart.add(pnlsss, BorderLayout.EAST);

    tsStart = getTimestamp( _START );

    // .......................................................
    // ...................... Date End .......................
    // .......................................................

    iYear  = 0;
    iMonth = 0;

    if (stData != null)
    {
	  setDate(stData.tsEnd);
	}

    pnlDateTimeEnd.setLayout(new GridLayout());
    pnlDateTimeEnd.setBorder(titledBorderEndDate);

    // -- Date End Panel

    pnlDateEnd.setLayout(new BorderLayout());

    // .. Year Textfield Panel

    pnlYYYYe.setLayout(new BorderLayout());
    pnlYYYYe.setBorder(titledBorderYYYY);
    pnlYYYYe.add(txtYYYYe,  BorderLayout.CENTER);

    sText = new String("2015");
    if (iYear > 0) { sText = new String(""+iYear); }

    txtYYYYe.setText( sText );
    txtYYYYe.setColumns(4);
	txtYYYYe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtYYYYe.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Month Textfield Panel

    pnlMMe.setLayout(new BorderLayout());
    pnlMMe.setBorder(titledBorderMM);
    pnlMMe.add(txtMMe,  BorderLayout.CENTER);

    sText = new String("12");
    if (iMonth > 0)
    {
	  iMonth++;
	  if (iMonth > 9)
	    sText = new String("" + iMonth);
	  else
	    sText = new String("0" + iMonth);
    }

    txtMMe.setText( sText );
    txtMMe.setColumns(2);
	txtMMe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMMe.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Day Textfield Panel

    pnlDDe.setLayout(new BorderLayout());
    pnlDDe.setBorder(titledBorderDD);
    pnlDDe.add(txtDDe,  BorderLayout.CENTER);

    txtDDe.setText("01");
    txtDDe.setEditable(false);
    txtDDe.setColumns(2);
	txtDDe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtDDe.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // -- Attach YYYY,MM,DD Panels to Date Start Panel

    pnlDateEnd.add(pnlYYYYe, BorderLayout.WEST);
    pnlDateEnd.add(pnlMMe,   BorderLayout.CENTER);
    pnlDateEnd.add(pnlDDe,   BorderLayout.EAST);

    // -- Time End Panel

    pnlTimeEnd.setLayout(new BorderLayout());

    // .. Hour Textfield Panel

    pnlHHe.setLayout(new BorderLayout());
    pnlHHe.setBorder(titledBorderHH);
    pnlHHe.add(txtHHe, BorderLayout.CENTER);

    txtHHe.setText("00");
    txtHHe.setEditable(false);
    txtHHe.setColumns(2);
	txtHHe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtHHe.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Minutes Textfield Panel

    pnlmme.setLayout(new BorderLayout());
    pnlmme.setBorder(titledBordermm);
    pnlmme.add(txtmme, BorderLayout.CENTER);

    txtmme.setText("00");
    txtmme.setEditable(false);
    txtmme.setColumns(2);
	txtmme.setHorizontalAlignment(SwingConstants.TRAILING);
	txtmme.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Seconds Textfield Panel

    pnlsse.setLayout(new BorderLayout());
    pnlsse.setBorder(titledBorderssS);
    pnlsse.add(txtssSe, BorderLayout.CENTER);

    txtssSe.setText("00.0");
    txtssSe.setEditable(false);
    txtssSe.setColumns(4);
	txtssSe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtssSe.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // -- Attach HH,mm,ss.SS Panels to Date Start Panel

    pnlTimeEnd.add(pnlHHe, BorderLayout.WEST);
    pnlTimeEnd.add(pnlmme, BorderLayout.CENTER);
    pnlTimeEnd.add(pnlsse, BorderLayout.EAST);

    tsEnd = getTimestamp( _END );

    // Attach Widgets

    panel.add(pnlXAxis,                BorderLayout.CENTER);
//    pnlXAxis.add(txtXAxis,                BorderLayout.NORTH);

//    pnlXAxis.add(pnlXAxisRange,           BorderLayout.SOUTH);

    pnlXAxis.add(pnlDateTimeStart,     null);
    pnlDateTimeStart.add(pnlDateStart, null);
    pnlDateTimeStart.add(pnlTimeStart, null);

    pnlXAxis.add(pnlDateTimeEnd,       null);
    pnlDateTimeEnd.add(pnlDateEnd,     null);
    pnlDateTimeEnd.add(pnlTimeEnd,     null);

    return (panel);
  }

  /** Method plotYAxisPanel()
   *  <p> This method will build the Y-Axis Panel
   */

  private JPanel plotYAxisPanel()
  {
	JPanel panel           = new JPanel();
    JPanel pnlYAxis        = new JPanel();
    JPanel pnlYAxisRange   = new JPanel();
    JPanel pnlYMin         = new JPanel();
    JPanel pnlYMax         = new JPanel();
    JPanel pnlYIncr        = new JPanel();

    JLabel lblYAxis        = new JLabel();
    JLabel lblYMin         = new JLabel();
    JLabel lblYMax         = new JLabel();
    JLabel lblYIncr        = new JLabel();

    TitledBorder titledBorderYAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Y-Axis:");
    titledBorderYAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMin = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum:");
    titledBorderMin.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum:");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderXIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Increment");
    titledBorderXIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    panel.setLayout(new BorderLayout());

    // .. Y-Axis Panel

    pnlYAxis.setLayout(new BorderLayout());
    pnlYAxis.setBorder(titledBorderYAxis);

    txtYAxis.setText( "Earthquake Magnitude" );
    txtYAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtYAxis.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Y-Axis Range Panel

    pnlYAxisRange.setLayout(new GridLayout());

    // .. .. Y-Axis Minimum Panel

    pnlYMin.setLayout(new BorderLayout());
    pnlYMin.setBorder(titledBorderMin);

    lblYMin.setText("Maximum: ");

    txtYMin.setText("0.0");
    txtYMin.setColumns(6);
    txtYMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMin.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. .. Y-Axis Maximum Panel

    pnlYMax.setLayout(new BorderLayout());
    pnlYMax.setBorder(titledBorderMax);

    lblYMax.setText(" Maximum: ");

    txtYMax.setText("6.0");
    txtYMax.setColumns(6);
    txtYMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMax.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. .. Y-Axis Increment Panel

    pnlYIncr.setLayout(new BorderLayout());
    pnlYIncr.setBorder(titledBorderXIncr);

    lblYIncr.setText(" " + sYIncr +": ");

    txtYIncr.setText("1.0");
    txtYIncr.setColumns(6);
    txtYIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYIncr.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // Attach Widgets to Panel

    panel.add(pnlYAxis,         BorderLayout.CENTER);
    pnlYAxis.add(txtYAxis,      BorderLayout.NORTH);

    pnlYAxis.add(pnlYAxisRange, BorderLayout.SOUTH);
    pnlYAxisRange.add(pnlYMin,  null);
//    pnlYMin.add(lblYMin,        BorderLayout.WEST);
    pnlYMin.add(txtYMin,        BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYMax,  null);
//    pnlYMax.add(lblYMax,        BorderLayout.WEST);
    pnlYMax.add(txtYMax,        BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYIncr, null);
//    pnlYIncr.add(lblYIncr,      BorderLayout.WEST);
    pnlYIncr.add(txtYIncr,      BorderLayout.CENTER);

    return (panel);
  }


  /** Method plotAxesPanel()
   *  <p> This method will build the Plot Axis Panels
   *

  private JPanel plotAxesPanel()
  {
	JPanel panel           = new JPanel();
    JPanel pnlTop          = new JPanel();

    JPanel pnlAxis         = new JPanel();
    JPanel pnlXAxis        = new JPanel();
    JPanel pnlXAxisRange   = new JPanel();
    JPanel pnlXMin         = new JPanel();
    JPanel pnlXMax         = new JPanel();
    JPanel pnlXIncr        = new JPanel();

    JPanel pnlYAxis        = new JPanel();
    JPanel pnlYAxisRange   = new JPanel();
    JPanel pnlYMin         = new JPanel();
    JPanel pnlYMax         = new JPanel();
    JPanel pnlYIncr        = new JPanel();

    JLabel lblXAxis        = new JLabel();
    JLabel lblXMin         = new JLabel();
    JLabel lblXMax         = new JLabel();
    JLabel lblXIncr        = new JLabel();

    JLabel lblYAxis        = new JLabel();
    JLabel lblYMin         = new JLabel();
    JLabel lblYMax         = new JLabel();
    JLabel lblYIncr        = new JLabel();

    String sXAxis          = "";
    double dXMin           = 0.0;
    double dXMax           = 0.0;
    double dXIncr          = 0.0;

    String sYAxis          = "";
    double dYMin           = 0.0;
    double dYMax           = 0.0;
    double dYIncr          = 0.0;

    TitledBorder titledBorderMin = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum:");
    titledBorderMin.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum:");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderXIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        sXIncr);
    titledBorderXIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderYIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        sYIncr);
    titledBorderYIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderXAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "X-Axis:");
    titledBorderXAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderYAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Y-Axis:");
    titledBorderYAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    if (iPlot > 0)
    {
	  if (stPlot[0] != null)
	  {
		sXAxis = new String(stPlot[0].sXAxis);
		dXMin  = stPlot[0].dXMinimum;
		dXMax  = stPlot[0].dXMaximum;
		dXIncr = stPlot[0].dXIncrement;
		if (stPlot[0].iXLog == 1)
		  dXIncr = stPlot[0].iXCycles;

		sYAxis = new String(stPlot[0].sYAxis);
		dYMin  = stPlot[0].dYMinimum;
		dYMax  = stPlot[0].dYMaximum;
		dYIncr = stPlot[0].dYIncrement;
		if (stPlot[0].iYLog == 1)
		  dYIncr = stPlot[0].iYCycles;
	  }
	}

    // Top Panel

    panel.setLayout(new BorderLayout());

    // -- Axis Panel

    pnlAxis.setLayout(new GridLayout());

    // .. X-Axis Panel

    pnlXAxis.setLayout(new BorderLayout());
    pnlXAxis.setBorder(titledBorderXAxis);

//    lblXAxis.setText( "X-Axis" );

    txtXAxis.setText( sXAxis );
    txtXAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtXAxis.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. X-Axis Range Panel

    pnlXAxisRange.setLayout(new GridLayout());

    // .. .. X-Axis Minimum Panel

    pnlXMin.setLayout(new BorderLayout());
//    pnlXMin.setBorder(titledBorderMin);

    lblXMin.setText("Minimum: ");

    txtXMin.setText("" + dXMin);
    txtXMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXMin.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. .. X-Axis Maximum Panel

    pnlXMax.setLayout(new BorderLayout());
//    pnlXMax.setBorder(titledBorderMax);

    lblXMax.setText(" Maximum: ");

    txtXMax.setText("" + dXMax);
    txtXMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXMax.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. .. X-Axis Increment Panel

    pnlXIncr.setLayout(new BorderLayout());
//    pnlXIncr.setBorder(titledBorderXIncr);

    lblXIncr.setText(" " + sXIncr +": ");

    txtXIncr.setText("" + dXIncr);
    txtXIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXIncr.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    if (iXLog == 0)
      txtXMax.setEnabled(true);
    else
      txtXMax.setEnabled(false);

    // .. Y-Axis Panel

    pnlYAxis.setLayout(new BorderLayout());
    pnlYAxis.setBorder(titledBorderYAxis);

    txtYAxis.setText( sYAxis );
    txtYAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtYAxis.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. Y-Axis Range Panel

    pnlYAxisRange.setLayout(new GridLayout());

    // .. .. Y-Axis Minimum Panel

    pnlYMin.setLayout(new BorderLayout());
//    pnlYMin.setBorder(titledBorderMin);

    lblYMin.setText("Maximum: ");

    txtYMin.setText("" + dYMin);
    txtYMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMin.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. .. Y-Axis Maximum Panel

    pnlYMax.setLayout(new BorderLayout());
//    pnlYMax.setBorder(titledBorderMax);

    lblYMax.setText(" Maximum: ");

    txtYMax.setText("" + dYMax);
    txtYMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMax.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    // .. .. Y-Axis Increment Panel

    pnlYIncr.setLayout(new BorderLayout());
//    pnlYIncr.setBorder(titledBorderYIncr);

    lblYIncr.setText(" " + sYIncr +": ");

    txtYIncr.setText("" + dYIncr);
    txtYIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYIncr.addFocusListener(new quakePlotTimeMagDataFrameFocusAdapter(this));

    if (iYLog == 0)
      txtYMax.setEnabled(true);
    else
      txtYMax.setEnabled(false);

    // Attach Widgets

    panel.add(pnlAxis,                    BorderLayout.NORTH);

    pnlAxis.add(pnlXAxis,                 null);
    pnlXAxis.add(txtXAxis,                BorderLayout.NORTH);

    pnlXAxis.add(pnlXAxisRange,           BorderLayout.SOUTH);
    pnlXAxisRange.add(pnlXMin,            null);
    pnlXMin.add(lblXMin,                  BorderLayout.WEST);
    pnlXMin.add(txtXMin,                  BorderLayout.CENTER);

    pnlXAxisRange.add(pnlXMax,            null);
    pnlXMax.add(lblXMax,                  BorderLayout.WEST);
    pnlXMax.add(txtXMax,                  BorderLayout.CENTER);

    pnlXAxisRange.add(pnlXIncr,           null);
    pnlXIncr.add(lblXIncr,                BorderLayout.WEST);
    pnlXIncr.add(txtXIncr,                BorderLayout.CENTER);

    pnlAxis.add(pnlYAxis,                 null);
    pnlYAxis.add(txtYAxis,                BorderLayout.NORTH);

    pnlYAxis.add(pnlYAxisRange,           BorderLayout.SOUTH);
    pnlYAxisRange.add(pnlYMin,            null);
    pnlYMin.add(lblYMin,                  BorderLayout.WEST);
    pnlYMin.add(txtYMin,                  BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYMax,            null);
    pnlYMax.add(lblYMax,                  BorderLayout.WEST);
    pnlYMax.add(txtYMax,                  BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYIncr,           null);
    pnlYIncr.add(lblYIncr,                BorderLayout.WEST);
    pnlYIncr.add(txtYIncr,                BorderLayout.CENTER);

    return (panel);
  }
*/
  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  /** METHOD quakePlotTimeMagDataFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class quakePlotTimeMagDataFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  if (notifier != null)
        notifier.notifyObservers(new String("Close Plot Control"));
    }
  }

  /** Method close()
   *  <p> This method will deallocate the global variables to force Java
   *      to free allocated memory.
   */

  public void close()
  {
//    stPlot    = null;
    stData   = null;

    if (panel != null)
      panel.close();
    panel     = null;

    if (pSearch != null)
      pSearch.close();
    pSearch   = null;

    tsStart   = null; // Starting Timestamp
    tsEnd     = null; // Ending Timestamp

    sPath     = null;
    sName     = null;

    sTitle    = null;
    sXAxis    = null;
    sXIncr    = null;
    sYAxis    = null;
    sYIncr    = null;

    // Global Frame Widgets

    // .. Plot Titles

    txtTitle1 = null;
    txtTitle2 = null;
    txtTitle3 = null;

    // .. X-Axis Widgets

    txtXAxis  = null;
//    txtXMin   = null;
//    txtXMax   = null;
//    txtXIncr  = null;

    // .. Y-Axis Widgets

    txtYAxis  = null;
    txtYMin   = null;
    txtYMax   = null;
    txtYIncr  = null;

    // .. Start Date & Time

    txtYYYYs    = null; // Year Field
    txtMMs      = null; // Month Field
    txtDDs      = null; // Day Field

    txtHHs      = null; // 24 Hour Field
    txtmms      = null; // minute Field
    txtssSs     = null; // second Field

    // .. End Date & Time

    txtYYYYe    = null; // Year Field
    txtMMe      = null; // Month Field
    txtDDe      = null; // Day Field

    txtHHe      = null; // 24 Hour Field
    txtmme      = null; // minute Field
    txtssSe     = null; // second Field

    // -- Menus

    mPDF      = null; // PDF of Profile Plot
    mExit     = null; // Exit

    dispose();
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (panel != null) image = panel.getImage();

    return (image);
  }

  /** Method getTimestamp()
   *  <p> This method will convert the Date Time String into a Timestamp.
   *  @param iDate = Start (0) or End (1)
   */

  private Timestamp getTimestamp( int iDate )
  {
	Timestamp        ts    = null;
    SimpleDateFormat dfm   = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
    String           sText = "";
    long             ltime = (long) 0;

    switch (iDate)
    {
      case _START:
        sText   = new String( txtYYYYs.getText() + "-" +
		                      txtMMs.getText()   + "-" +
		                      txtDDs.getText()   + " " +
		                      txtHHs.getText()   + ":" +
		                      txtmms.getText()   + ":" +
		                      txtssSs.getText() );
        break;

      case _END:
        sText   = new String( txtYYYYe.getText() + "-" +
  		                      txtMMe.getText()   + "-" +
		                      txtDDe.getText()   + " " +
		                      txtHHe.getText()   + ":" +
		                      txtmme.getText()   + ":" +
		                      txtssSe.getText() );
        break;
    }

    try
    {
      ltime = dfm.parse(sText).getTime();
      ts    = new Timestamp( ltime );
	}
    catch (ParseException e)
    {
      e.printStackTrace();
    }

	return (ts);
  }

  /* ================================================================== *
   * -------------------------- SET METHODS --------------------------- *
   * ================================================================== */

  /** Method setDate()
   *  <p> This method will set the Date Value into either the
   *      Start Date or End Date.
   *  @param ts    = timestamp
   */

  private void setDate(Timestamp ts)
  {
    int i = 0;
    String tokens[]    = null;
    String sDelimiter  = new String("[ ]+");
    String sDelimiter1 = new String("[-]+");
    String str         = "";
    String sDate       = "";
    String date[]      = { "", "", "" };
    int    iContinue   = 0;
    double d           = 0.0;
    int    id          = 0;

    if (ts != null)
    {
      str = new String(ts.toString());
      tokens = str.split( sDelimiter );
      if (tokens.length > 0)
        sDate = new String(tokens[0].trim());

      tokens = sDate.split( sDelimiter1 );
      if (tokens.length == 3)
      {
  	    for (i=0; i<3; i++)
	    {
	      date[i] = new String(tokens[i]);
	    }

        iYear  = cmn.cmnString.stringToInt(date[0]);
	    iMonth = cmn.cmnString.stringToInt(date[1]);
      }
    }
  }

  /** Method refresh()
   * <p> This method will refresh the plot if it exists
   */

  public void refresh() { panel.repaint(); }

  /** Method setLegend()
   *  <p> The Legend for the XY Plot.
   *  @param iLegend  - Number of Legend Rows.
   *  @param stLegend - Array of Legends.
   */

  public void setLegend(int iLegend, plotSymbolStruct stLegend[])
  {
    panel.setLegend(iLegend, stLegend);
  }

  /** Method setFlag()
   *  <p> Set a Flag to filter extra data or not.
   *  @param bFlag = true to filter data or false not to filter extra data.
   */

  public void setFlag(boolean bFlag) { panel.setFlag(bFlag); }

  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText)
  {
	txtTitle1.setText(sText);
	panel.setTitle1(sText);
  }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText)
  {
	txtTitle2.setText(sText);
	panel.setTitle2(sText);
  }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText)
  {
	txtTitle3.setText(sText);
	panel.setTitle3(sText);
  }

  /** Method setXAxis()
   *  @param tsStart = starting Date & Time
   *  @param tsEnd   = ending Data & Time
   */

  public void setXAxis( Timestamp tsStart, Timestamp tsEnd )
  {
    panel.setXAxis(tsStart, tsEnd);
  }

  /** Method setXAxis()
   * <p> This method will reset the X-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setXAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
//    panel.setXAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setYAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setData()
   *  <p> This method will set the data to be plotted
   *  @param iData = The data type to be plotted
   */

  public void setData(int iData) { panel.setData(iData); }

  /* ===================================================================== *
   * -------------------------- PDF ACTIONS ------------------------------ *
   * ===================================================================== */

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Production Plot
   */

  private void createPDF()
  {
    quake.io.quakeIO_Files.createImage( sPath, sName, panel.getImage() );
    quake.io.quakeIO_Files.createHTML( sPath, sName );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == mPDF) // Create a PDF of Profile Plot
    {
	  if (sPath.length() == 0)
        sPath = quake.io.quakeIO_Files.getHomeDir();

      if (sName.length() == 0)
        sName = new String( "KS_earthquakes" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".png");
    }

    if (event.getSource() == mExit)  { close(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	focusLostTitles(e);
	focusLostXAxes(e);
	focusLostYAxes(e);
  }

  /** METHOD focusLostTitles()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostTitles(FocusEvent e)
  {
    if (e.getSource() == txtTitle1)
    {
	  setTitle1(txtTitle1.getText());
	}

    if (e.getSource() == txtTitle2)
    {
	  setTitle2(txtTitle2.getText());
	}

    if (e.getSource() == txtTitle3)
    {
	  setTitle3(txtTitle3.getText());
	}
  }

  /** METHOD focusLostXAxes()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostXAxes(FocusEvent e)
  {
    int     iNumeric = 0;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     iField   = -1;
    int     iDate    = -1;
    int     id       = 0;
    int     len      = 0;
    boolean bVal     = true;

    if ((e.getSource() == txtYYYYs) ||
        (e.getSource() == txtMMs )  ||
        (e.getSource() == txtYYYYe) ||
        (e.getSource() == txtMMe))
    {
      if (e.getSource() == txtYYYYs )
      {
        iNumeric = 1;
        iDate    = _START;
        iField   = YEAR;
        sTemp    = txtYYYYs.getText();
        len      = sTemp.length();
        sMessage = new String("Start Year Value is a Numeric Field");
	  }

      if (e.getSource() == txtYYYYe )
      {
        iNumeric = 1;
        iDate    = _END;
        iField   = YEAR;
        sTemp    = txtYYYYe.getText();
        len      = sTemp.length();
        sMessage = new String("End Year Value is a Numeric Field");
	  }

      if (e.getSource() == txtMMs )
      {
        iNumeric = 1;
        iDate    = _START;
        iField   = MONTH;
        sTemp    = txtMMs.getText();
        len      = sTemp.length();
        sMessage = new String("Start Month Value is a Numeric Field");
	  }

      if (e.getSource() == txtMMe )
      {
        iNumeric = 1;
        iDate    = _END;
        iField   = MONTH;
        sTemp    = txtMMe.getText();
        len      = sTemp.length();
        sMessage = new String("End Month Value is a Numeric Field");
	  }

      if (iNumeric == 1)
      {
        if (!cmn.cmnString.isNumeric(sTemp))
        {
          JOptionPane.showMessageDialog((Component) null,
                                         sMessage,
                                         "ERROR",
                                         JOptionPane.ERROR_MESSAGE);
        }
        else
        {
		  id = cmn.cmnString.stringToInt(sTemp);

          switch (iDate)
          {
            case _START:
              switch (iField)
              {
                case YEAR:
                  if (len != 4) { bVal = false; }
                  break;
                case MONTH:
                  if (len > 2) { bVal = false; }
                  else if ((id < 0) || (id >= 12)) { bVal = false; }
                  else if (len < 2) { txtMMs.setText("0"+sTemp); }
                  break;
	          }
              break;

            case _END:
  	          switch (iField)
              {
                case YEAR:
                  if (len != 4) { bVal = false; }
                    bVal = false;
                  break;
                case MONTH:
                  if (len > 2) { bVal = false; }
                  else if ((id < 0) || (id >= 12)) { bVal = false; }
                  else if (len < 2) { txtMMe.setText("0"+sTemp); }
                  break;
	          }
	          break;
		  }
		}
	  }

	  if (bVal)
	  {
        tsStart = getTimestamp( _START );
        tsEnd   = getTimestamp( _END );
        setXAxis( tsStart, tsEnd );
	  }
	}
  }

  /** METHOD focusLostXAxes()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   *

  protected void focusLostXAxes(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");
    String s1       = "";
    String s2       = "";
    String s3       = "";
    double d1       = 0.0;
    double d2       = 0.0;
    double dTemp    = 0.0;
    int    iMin     = 0;
    int    iMax     = 0;
    int    iDiff    = 0;
    double dMin     = 0.0;
    double dMax     = 0.0;
    double diff     = 0.0;

    if ((e.getSource() == txtXAxis) ||
        (e.getSource() == txtXMin ) ||
        (e.getSource() == txtXMax ) ||
        (e.getSource() == txtXIncr))
    {
      if (e.getSource() == txtXAxis)
      {
        sXAxis = new String( txtXAxis.getText() );
        setXAxis( sXAxis, iXLog, dXMaximum, dXMinimum, dXIncrement, iXCycles );
      }

      if (e.getSource() == txtXMin )
      {
        iNumeric = 1;
        sTemp    = txtXMin.getText();
        sMessage = new String("X-Minimum Value is a Numeric Field");

        s1 = txtXMin.getText();
        s2 = txtXMax.getText();

        if (s1.equals(s2))
        {
          txtXMin.setText(""+dXMinimum);

	      sTemp = new String("A");
          sMessage = new String("X-Minimum Value can not equal X-Maximum Value");
        }

        if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
        {
	      d1 = cmn.cmnString.stringToDouble( s1 );
	      d2 = cmn.cmnString.stringToDouble( s2 );

          if (d1 > d2 )
      	  {
      	    txtXMin.setText(""+dXMinimum);

    	    sTemp = new String("A");
            sMessage = new String("X-Minimum Value can not be greater than X-Maximum Value");
    	  }
        }
      }

      if (e.getSource() == txtXMax )
      {
        iNumeric = 1;
        sTemp    = txtXMax.getText();
        sMessage = new String("X-Maximum Value is a Numeric Field");

        s1 = txtXMin.getText();
        s2 = txtXMax.getText();

        if (s1.equals(s2))
        {
	      txtXMax.setText(""+dXMaximum);

	      sTemp = new String("A");
          sMessage = new String("X-Minimum Value can not equal X-Maximum Value");
        }

        if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
        {
	      d1 = cmn.cmnString.stringToDouble( s1 );
	      d2 = cmn.cmnString.stringToDouble( s2 );

	      if (d1 > d2 )
	      {
	        txtXMax.setText(""+dXMaximum);

	        sTemp = new String("A");
            sMessage = new String("X-Minimum Value can not be greater than X-Maximum Value");
      	  }
        }
      }

      if (e.getSource() == txtXIncr)
      {
        iNumeric = 1;
        sTemp    = txtXIncr.getText();
        sMessage = new String("X-Increment Value is a Numeric Field");

        s3 = txtXIncr.getText();
        if ((s3.equals("0.0")) || (s3.equals("0")))
        {
    	  txtXIncr.setText(""+dXIncrement);
    	  if (iXLog == 1) { txtYIncr.setText(""+iXCycles); }
    	  sTemp = new String("A");
          sMessage = new String("X-Increment Value can not be 0");
        }
      }

      if (iNumeric == 1)
      {
        if (!cmn.cmnString.isNumeric(sTemp))
        {
          JOptionPane.showMessageDialog((Component) null,
                                         sMessage,
                                         "ERROR",
                                         JOptionPane.ERROR_MESSAGE);
        }
        else
        {
          if (e.getSource() == txtXMin )
          {
            dXMinimum = cmn.cmnString.stringToDouble(txtXMin.getText());

    	    if (iXLog == 1)
	 	    {
              dMin = Math.log(dXMinimum)/Math.log(10.0);
              if (dMin < 0.0) { dMin = dMin - 1.0; }
              iMin = (int) dMin;
              dXMinimum = Math.pow(10.0, iMin);
              txtXMin.setText(""+dXMinimum);

              iMax = iMin + iXCycles;
              dXMaximum = Math.pow(10.0, iMax);
              txtXMax.setText(""+dXMaximum);
	        }
          }

          if (e.getSource() == txtXMax )
            dXMaximum = cmn.cmnString.stringToDouble(txtXMax.getText());

          if (e.getSource() == txtXIncr )
          {
		    dTemp = cmn.cmnString.stringToDouble(txtXIncr.getText());
		    if (iXLog == 0)
		      dXIncrement = dTemp;
		    else
		    {
		      iXCycles    = (int) dTemp;
		      txtXIncr.setText(""+iXCycles);
	        }

		    if (iXLog == 1)
		    {
              iMin = (int) ( Math.log(dXMinimum)/Math.log(10.0) );
              iMax = iMin + iXCycles;
              dXMaximum = Math.pow(10.0, iMax);
              txtXMax.setText(""+dXMaximum);
            }
	      }

          setXAxis( sXAxis, iXLog, dXMaximum, dXMinimum, dXIncrement, iXCycles );
        }
	  }
	}
  }
*/
  /** METHOD focusLostYAxes()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostYAxes(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");
    String s1       = "";
    String s2       = "";
    String s3       = "";
    double d1       = 0.0;
    double d2       = 0.0;
    double dTemp    = 0.0;
    int    iMin     = 0;
    int    iMax     = 0;
    int    iDiff    = 0;
    double dMin     = 0.0;
    double dMax     = 0.0;
    double diff     = 0.0;

    if ((e.getSource() == txtYAxis) ||
        (e.getSource() == txtYMin ) ||
        (e.getSource() == txtYMax ) ||
        (e.getSource() == txtYIncr))
    {
      if (e.getSource() == txtYAxis)
      {
        sYAxis = new String( txtYAxis.getText() );
        setYAxis( sYAxis, iYLog, dYMaximum, dYMinimum, dYIncrement, iYCycles );
	  }

      if (e.getSource() == txtYMin )
      {
        iNumeric = 1;
        sTemp    = txtYMin.getText();
        sMessage = new String("Y-Minimum Value is a Numeric Field");

        s1 = txtYMin.getText();
        s2 = txtYMax.getText();

        if (s1.equals(s2))
        {
	      txtYMin.setText(""+dYMinimum);

	      sTemp = new String("A");
          sMessage = new String("Y-Minimum Value can not equal Y-Maximum Value");
	    }

	    if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	    {
	      d1 = cmn.cmnString.stringToDouble( s1 );
	      d2 = cmn.cmnString.stringToDouble( s2 );

	      if (d1 > d2 )
	      {
	     	  txtYMin.setText(""+dYMinimum);

	    	  sTemp = new String("A");
              sMessage = new String("Y-Minimum Value can not be greater than Y-Maximum Value");
	       }
	    }
	  }

      if (e.getSource() == txtYMax )
      {
        iNumeric = 1;
        sTemp    = txtYMax.getText();
        sMessage = new String("Y-Maximum Value is a Numeric Field");

        s1 = txtYMin.getText();
        s2 = txtYMax.getText();

        if (s1.equals(s2))
        {
	      txtYMax.setText(""+dYMaximum);

	      sTemp = new String("A");
          sMessage = new String("Y-Minimum Value can not equal Y-Maximum Value");
	    }

	    if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	    {
	      d1 = cmn.cmnString.stringToDouble( s1 );
	      d2 = cmn.cmnString.stringToDouble( s2 );

	      if (d1 > d2 )
	      {
	        txtYMax.setText(""+dYMaximum);

	        sTemp = new String("A");
            sMessage = new String("Y-Minimum Value can not be greater than Y-Maximum Value");
	      }
	    }
	  }

      if (e.getSource() == txtYIncr)
      {
        iNumeric = 1;
        sTemp    = txtYIncr.getText();
        sMessage = new String("Y-Increment Value is a Numeric Field");

        s3 = txtYIncr.getText();
	    if ((s3.equals("0.0")) || (s3.equals("0")))
	    {
	   	  txtYIncr.setText(""+dYIncrement);
	      if (iYLog == 1) { txtYIncr.setText(""+iYCycles); }
	   	  sTemp = new String("A");
          sMessage = new String("Y-Increment Value can not be 0");
	    }
	  }

      if (iNumeric == 1)
      {
        if (!cmn.cmnString.isNumeric(sTemp))
        {
          JOptionPane.showMessageDialog((Component) null,
                                         sMessage,
                                         "ERROR",
                                         JOptionPane.ERROR_MESSAGE);
        }
        else
        {
          if ((e.getSource() == txtYMin ) ||
              (e.getSource() == txtYMax ) ||
              (e.getSource() == txtYIncr ))
          {
            if (e.getSource() == txtYMin )
            {
              dYMinimum = cmn.cmnString.stringToDouble(txtYMin.getText());

	   	      if (iYLog == 1)
	 	      {
                dMin = Math.log(dYMinimum)/Math.log(10.0);
                if (dMin < 0.0) { dMin = dMin - 1.0; }
                iMin = (int) dMin;
                dYMinimum = Math.pow(10.0, iMin);
                txtYMin.setText(""+dYMinimum);

                iMax = iMin + iYCycles;
                dYMaximum = Math.pow(10.0, iMax);
                txtYMax.setText(""+dYMaximum);
	          }
	        }

            if (e.getSource() == txtYMax )
              dYMaximum = cmn.cmnString.stringToDouble(txtYMax.getText());

            if (e.getSource() == txtYIncr )
            {
	          dTemp = cmn.cmnString.stringToDouble(txtYIncr.getText());
		      if (iYLog == 0)
		        dYIncrement = dTemp;
		      else
		      {
		        iYCycles    = (int) dTemp;
		        txtYIncr.setText(""+iYCycles);
	          }

		      if (iYLog == 1)
		      {
                iMin = (int) ( Math.log(dYMinimum)/Math.log(10.0) );
                iMax = iMin + iYCycles;
                dYMaximum = Math.pow(10.0, iMax);
                txtYMax.setText(""+dYMaximum);
	          }
	        }

            setYAxis( sYAxis, iYLog, dYMaximum, dYMinimum, dYIncrement, iYCycles );
		  }
        }
	  }
	}
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      createPDF();

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}
  }
}

/** CLASS quakePlotTimeMagDataFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class quakePlotTimeMagDataFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  quakePlotTimeMagDataFrame adaptee;

  quakePlotTimeMagDataFrameFocusAdapter(quakePlotTimeMagDataFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/** CLASS quakePlotTimeMagDataFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class quakePlotTimeMagDataFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 08/06/2015
 *  @author  John Victorine
 */
