/*
 * @quakeTable.java Version 1.1 02/11/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.sql.Timestamp;

import gui.guiTable;

import quake.quakeSeisStruct;  // Seismic Data Structure

import plot.plotXYStruct;      // Plot XY Data Structure

/** Class quakeTable
 *  <p> This Class will display the Headers Status Table.
 *
 *  @version 1.1 02/11/2016
 *  @author  John R. Victorine
 */

public class quakeTable
{
  private int      iRows      = 0;
  private int      iColumns   = 6;
  private String   sColumn[]  = {"Date",
                                 "Magnitude",
                                 "Latitude",
                                 "Longitude",
                                 "Depth (km)",
                                 "Source"};
  private int      len        = 0;
  private String   sTable[][] = null;
  private Object   oData[][]  = null;
  private guiTable pTable     = null;
  private int      iChange    = 0;

  /** Constructor quakeTable()
   *  <p> This is the Constructor for this class.
   */

  public quakeTable(quakeSeisStruct st, plotXYStruct stPo)
  {
	getTableData( st, stPo);
    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiTable(0, iColumns, sColumn);

    if (iChange == 0)
      iRows = 0;

    pTable.resizeColumn(0, 20);
    pTable.resizeColumn(1,  6);
    pTable.resizeColumn(2,  8);
    pTable.resizeColumn(3,  8);
    pTable.resizeColumn(4,  6);
    pTable.resizeColumn(5, 15);

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /** Method close()
   * <p> This method will set global variables to null in order to force
   *     Java to deallocate memory
   */

  public void close()
  {
    sColumn    = null;
    sTable     = null;
    oData      = null;
    pTable     = null;
  }

  /** Method populateList
   *  <p> This method will load the Data Array for the guiTable.
   */

  void populateList()
  {
    iRows = 0;

    if (len > 0)
    {
	  iChange = 1;
      oData   = new Object[len][iColumns];

      for (int i=0; i<len; i++)
      {
	    for (int j=0; j<iColumns; j++)
	    {
          oData[i][j] = new String( sTable[i][j] );
	    }

        iRows++;
	  }
    }
    else
    {
	  iChange = 0;
      oData = new Object[1][iColumns];
      for (int j=0; j<iColumns; j++)
      {
        oData[0][j] = new String( "" );
      }
      iRows = 1;
	}
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  public void setData(quakeSeisStruct st, plotXYStruct stPo)
  {
	getTableData( st, stPo);
    populateList();

    if (iChange == 0)
      iRows = 0;

    pTable.reload(iRows, oData);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getTableData()
   *  <p> This method will build the Plot XY List Data Structure
   *      to create the effect of animation of the Earthquake data.
   *  @param  st     = Seismic Data Structure
   *  @param  stPo   = XY Plot Structure
   */

  private void getTableData(quakeSeisStruct st, plotXYStruct stPo)
  {
	int     i=0;
	int     icnt  = 0;
	int     ittl  = 0;

	if (st != null)
	{
	  if (st.iRows > 0)
	  {
		for (i=0; i<st.iRows; i++)
		{
          if (quake.quakeAnimateUtility.isDepth( st.depth[i], st.mag[i], stPo ))
          {
            ittl++;
          }
		}

        sTable = new String[ittl][6];
        icnt   = 0;

		for (i=0; i<st.iRows; i++)
		{
          if (quake.quakeAnimateUtility.isDepth( st.depth[i], st.mag[i], stPo ))
          {
			if (icnt < ittl)
			{
			  sTable[icnt][0] = new String( st.ts[i].toString() );
			  sTable[icnt][1] = new String( "" + st.mag[i] );
			  sTable[icnt][2] = new String( "" + st.dLatitude[i] );
			  sTable[icnt][3] = new String( "" + st.dLongitude[i] );
			  sTable[icnt][4] = new String( "" + st.depth[i] );
			  sTable[icnt][5] = new String( st.src[i] );
			  icnt++;
		    }
		  }
		}
	  }

	  len = icnt;
	}
  }

  /** Method getSelectedRow()
   *  <p> This method will return the selected row.
   *  @return iSelectedRow - Selected Row.
   */

  public int getSelectedRow() { return (pTable.getSelectedRow()); }

  /** Method getTotalRows()
   *  <p> This method will return the total number of rows.
   *  @return iRows - Total Rows.
   */

  public int getTotalRows() { return (iRows); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}

/*
 *  @version 1.1 02/11/2016
 *  @author  John Victorine
 */
